@@ -22,12 +22,12 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define FC_NUM_CASE_FOLD	264
+#define FC_NUM_CASE_FOLD	271
 #define FC_NUM_CASE_FOLD_CHARS	471
 #define FC_MAX_CASE_FOLD_CHARS	6
 #define FC_MAX_CASE_FOLD_EXPAND	4
 #define FC_MIN_FOLD_CHAR	0x00000041
-#define FC_MAX_FOLD_CHAR	0x00010427
+#define FC_MAX_FOLD_CHAR	0x000118bf
 
 static const FcCaseFold    fcCaseFold[FC_NUM_CASE_FOLD] = {
     { 0x00000041, FC_CASE_FOLD_RANGE,    0x001a,     32 },
@@ -101,6 +101,7 @@ static const FcCaseFold    fcCaseFold[FC_NUM_CASE_FOLD] = {
     { 0x00000345, FC_CASE_FOLD_RANGE,    0x0001,    116 },
     { 0x00000370, FC_CASE_FOLD_EVEN_ODD, 0x0003,      1 },
     { 0x00000376, FC_CASE_FOLD_EVEN_ODD, 0x0001,      1 },
+    { 0x0000037f, FC_CASE_FOLD_RANGE,    0x0001,    116 },
     { 0x00000386, FC_CASE_FOLD_RANGE,    0x0001,     38 },
     { 0x00000388, FC_CASE_FOLD_RANGE,    0x0003,     37 },
     { 0x0000038c, FC_CASE_FOLD_RANGE,    0x0001,     64 },
@@ -130,7 +131,7 @@ static const FcCaseFold    fcCaseFold[FC_NUM_CASE_FOLD] = {
     { 0x0000048a, FC_CASE_FOLD_EVEN_ODD, 0x0035,      1 },
     { 0x000004c0, FC_CASE_FOLD_RANGE,    0x0001,     15 },
     { 0x000004c1, FC_CASE_FOLD_EVEN_ODD, 0x000d,      1 },
-    { 0x000004d0, FC_CASE_FOLD_EVEN_ODD, 0x0057,      1 },
+    { 0x000004d0, FC_CASE_FOLD_EVEN_ODD, 0x005f,      1 },
     { 0x00000531, FC_CASE_FOLD_RANGE,    0x0026,     48 },
     { 0x00000587, FC_CASE_FOLD_FULL,     0x0004,     23 },
     { 0x000010a0, FC_CASE_FOLD_RANGE,    0x0026,   7264 },
@@ -269,7 +270,7 @@ static const FcCaseFold    fcCaseFold[FC_NUM_CASE_FOLD] = {
     { 0x00002ceb, FC_CASE_FOLD_EVEN_ODD, 0x0003,      1 },
     { 0x00002cf2, FC_CASE_FOLD_EVEN_ODD, 0x0001,      1 },
     { 0x0000a640, FC_CASE_FOLD_EVEN_ODD, 0x002d,      1 },
-    { 0x0000a680, FC_CASE_FOLD_EVEN_ODD, 0x0017,      1 },
+    { 0x0000a680, FC_CASE_FOLD_EVEN_ODD, 0x001b,      1 },
     { 0x0000a722, FC_CASE_FOLD_EVEN_ODD, 0x000d,      1 },
     { 0x0000a732, FC_CASE_FOLD_EVEN_ODD, 0x003d,      1 },
     { 0x0000a779, FC_CASE_FOLD_EVEN_ODD, 0x0003,      1 },
@@ -278,8 +279,13 @@ static const FcCaseFold    fcCaseFold[FC_NUM_CASE_FOLD] = {
     { 0x0000a78b, FC_CASE_FOLD_EVEN_ODD, 0x0001,      1 },
     { 0x0000a78d, FC_CASE_FOLD_RANGE,    0x0001,  23256 },
     { 0x0000a790, FC_CASE_FOLD_EVEN_ODD, 0x0003,      1 },
-    { 0x0000a7a0, FC_CASE_FOLD_EVEN_ODD, 0x0009,      1 },
+    { 0x0000a796, FC_CASE_FOLD_EVEN_ODD, 0x0013,      1 },
     { 0x0000a7aa, FC_CASE_FOLD_RANGE,    0x0001,  23228 },
+    { 0x0000a7ab, FC_CASE_FOLD_RANGE,    0x0001,  23217 },
+    { 0x0000a7ac, FC_CASE_FOLD_RANGE,    0x0001,  23221 },
+    { 0x0000a7ad, FC_CASE_FOLD_RANGE,    0x0001,  23231 },
+    { 0x0000a7b0, FC_CASE_FOLD_RANGE,    0x0001,  23278 },
+    { 0x0000a7b1, FC_CASE_FOLD_RANGE,    0x0001,  23254 },
     { 0x0000fb00, FC_CASE_FOLD_FULL,     0x0002,    435 },
     { 0x0000fb01, FC_CASE_FOLD_FULL,     0x0002,    437 },
     { 0x0000fb02, FC_CASE_FOLD_FULL,     0x0002,    439 },
@@ -294,6 +300,7 @@ static const FcCaseFold    fcCaseFold[FC_NUM_CASE_FOLD] = {
     { 0x0000fb17, FC_CASE_FOLD_FULL,     0x0004,    467 },
     { 0x0000ff21, FC_CASE_FOLD_RANGE,    0x001a,     32 },
     { 0x00010400, FC_CASE_FOLD_RANGE,    0x0028,     40 },
+    { 0x000118a0, FC_CASE_FOLD_RANGE,    0x0020,     32 },
 };
 
 static const FcChar8	fcCaseFoldChars[FC_NUM_CASE_FOLD_CHARS] = {
