@@ -26,28 +26,14 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
-#ifdef HAVE_REGEX_H
-#include <regex.h>
-#endif
-#ifdef _WIN32
-#include <windows.h>
-#endif
+
+
+/* Objects MT-safe for readonly access. */
 
 FcChar8 *
 FcStrCopy (const FcChar8 *s)
 {
-    int     len;
-    FcChar8 *r;
-
-    if (!s)
-	return 0;
-    len = strlen ((char *) s) + 1;
-    r = (FcChar8 *) malloc (len);
-    if (!r)
-	return 0;
-    FcMemAlloc (FC_MEM_STRING, len);
-    memcpy (r, s, len);
-    return r;
+    return FcStrdup (s);
 }
 
 FcChar8 *
@@ -60,7 +46,6 @@ FcStrPlus (const FcChar8 *s1, const FcChar8 *s2)
 
     if (!s)
 	return 0;
-    FcMemAlloc (FC_MEM_STRING, l);
     memcpy (s, s1, s1l);
     memcpy (s + s1l, s2, s2l + 1);
     return s;
@@ -69,7 +54,6 @@ FcStrPlus (const FcChar8 *s1, const FcChar8 *s2)
 void
 FcStrFree (FcChar8 *s)
 {
-    FcMemFree (FC_MEM_STRING, strlen ((char *) s) + 1);
     free (s);
 }
 
@@ -79,8 +63,6 @@ FcStrFree (FcChar8 *s)
 #define FcCaseFoldUpperCount(cf) \
     ((cf)->method == FC_CASE_FOLD_FULL ? 1 : (cf)->count)
 
-#define FC_STR_CANON_BUF_LEN	1024
-
 typedef struct _FcCaseWalker {
     const FcChar8   *read;
     const FcChar8   *src;
@@ -152,27 +134,7 @@ FcStrCaseWalkerLong (FcCaseWalker *w, FcChar8 r)
 }
 
 static FcChar8
-FcStrCaseWalkerNext (FcCaseWalker *w)
-{
-    FcChar8	r;
-
-    if (w->read)
-    {
-	if ((r = *w->read++))
-	    return r;
-	w->read = 0;
-    }
-    r = *w->src++;
-
-    if ((r & 0xc0) == 0xc0)
-	return FcStrCaseWalkerLong (w, r);
-    if ('A' <= r && r <= 'Z')
-        r = r - 'A' + 'a';
-    return r;
-}
-
-static FcChar8
-FcStrCaseWalkerNextIgnoreBlanks (FcCaseWalker *w)
+FcStrCaseWalkerNext (FcCaseWalker *w, const char *delims)
 {
     FcChar8	r;
 
@@ -185,7 +147,7 @@ FcStrCaseWalkerNextIgnoreBlanks (FcCaseWalker *w)
     do
     {
 	r = *w->src++;
-    } while (r == ' ');
+    } while (r != 0 && delims && strchr (delims, r));
 
     if ((r & 0xc0) == 0xc0)
 	return FcStrCaseWalkerLong (w, r);
@@ -202,14 +164,13 @@ FcStrDowncase (const FcChar8 *s)
     FcChar8	    *dst, *d;
 
     FcStrCaseWalkerInit (s, &w);
-    while (FcStrCaseWalkerNext (&w))
+    while (FcStrCaseWalkerNext (&w, NULL))
 	len++;
     d = dst = malloc (len + 1);
     if (!d)
 	return 0;
-    FcMemAlloc (FC_MEM_STRING, len + 1);
     FcStrCaseWalkerInit (s, &w);
-    while ((*d++ = FcStrCaseWalkerNext (&w)));
+    while ((*d++ = FcStrCaseWalkerNext (&w, NULL)));
     return dst;
 }
 
@@ -226,8 +187,8 @@ FcStrCmpIgnoreCase (const FcChar8 *s1, const FcChar8 *s2)
 
     for (;;)
     {
-	c1 = FcStrCaseWalkerNext (&w1);
-	c2 = FcStrCaseWalkerNext (&w2);
+	c1 = FcStrCaseWalkerNext (&w1, NULL);
+	c2 = FcStrCaseWalkerNext (&w2, NULL);
 	if (!c1 || (c1 != c2))
 	    break;
     }
@@ -236,6 +197,12 @@ FcStrCmpIgnoreCase (const FcChar8 *s1, const FcChar8 *s2)
 
 int
 FcStrCmpIgnoreBlanksAndCase (const FcChar8 *s1, const FcChar8 *s2)
+{
+    return FcStrCmpIgnoreCaseAndDelims (s1, s2, (const FcChar8 *)" ");
+}
+
+int
+FcStrCmpIgnoreCaseAndDelims (const FcChar8 *s1, const FcChar8 *s2, const FcChar8 *delims)
 {
     FcCaseWalker    w1, w2;
     FcChar8	    c1, c2;
@@ -247,8 +214,8 @@ FcStrCmpIgnoreBlanksAndCase (const FcChar8 *s1, const FcChar8 *s2)
 
     for (;;)
     {
-	c1 = FcStrCaseWalkerNextIgnoreBlanks (&w1);
-	c2 = FcStrCaseWalkerNextIgnoreBlanks (&w2);
+	c1 = FcStrCaseWalkerNext (&w1, (const char *)delims);
+	c2 = FcStrCaseWalkerNext (&w2, (const char *)delims);
 	if (!c1 || (c1 != c2))
 	    break;
     }
@@ -272,55 +239,6 @@ FcStrCmp (const FcChar8 *s1, const FcChar8 *s2)
     return (int) c1 - (int) c2;
 }
 
-#ifdef USE_REGEX
-static FcBool
-_FcStrRegexCmp (const FcChar8 *s, const FcChar8 *regex, int cflags, int eflags)
-{
-    int ret = -1;
-    regex_t reg;
-
-    if ((ret = regcomp (&reg, (const char *)regex, cflags)) != 0)
-    {
-	if (FcDebug () & FC_DBG_MATCHV)
-	{
-	    char buf[512];
-
-	    regerror (ret, &reg, buf, 512);
-	    printf("Regexp compile error: %s\n", buf);
-	}
-	return FcFalse;
-    }
-    ret = regexec (&reg, (const char *)s, 0, NULL, eflags);
-    if (ret != 0)
-    {
-	if (FcDebug () & FC_DBG_MATCHV)
-	{
-	    char buf[512];
-
-	    regerror (ret, &reg, buf, 512);
-	    printf("Regexp exec error: %s\n", buf);
-	}
-    }
-    regfree (&reg);
-
-    return ret == 0 ? FcTrue : FcFalse;
-}
-#else
-#  define _FcStrRegexCmp(_s_, _regex_, _cflags_, _eflags_)	(FcFalse)
-#endif
-
-FcBool
-FcStrRegexCmp (const FcChar8 *s, const FcChar8 *regex)
-{
-	return _FcStrRegexCmp (s, regex, REG_EXTENDED | REG_NOSUB, 0);
-}
-
-FcBool
-FcStrRegexCmpIgnoreCase (const FcChar8 *s, const FcChar8 *regex)
-{
-	return _FcStrRegexCmp (s, regex, REG_EXTENDED | REG_NOSUB | REG_ICASE, 0);
-}
-
 /*
  * Return a hash value for a string
  */
@@ -333,7 +251,7 @@ FcStrHashIgnoreCase (const FcChar8 *s)
     FcChar8	    c;
 
     FcStrCaseWalkerInit (s, &w);
-    while ((c = FcStrCaseWalkerNext (&w)))
+    while ((c = FcStrCaseWalkerNext (&w, NULL)))
 	h = ((h << 3) ^ (h >> 3)) ^ c;
     return h;
 }
@@ -353,8 +271,8 @@ FcStrIsAtIgnoreBlanksAndCase (const FcChar8 *s1, const FcChar8 *s2)
 
     for (;;)
     {
-	c1 = FcStrCaseWalkerNextIgnoreBlanks (&w1);
-	c2 = FcStrCaseWalkerNextIgnoreBlanks (&w2);
+	c1 = FcStrCaseWalkerNext (&w1, " ");
+	c2 = FcStrCaseWalkerNext (&w2, " ");
 	if (!c1 || (c1 != c2))
 	    break;
     }
@@ -412,8 +330,8 @@ FcStrIsAtIgnoreCase (const FcChar8 *s1, const FcChar8 *s2)
 
     for (;;)
     {
-	c1 = FcStrCaseWalkerNext (&w1);
-	c2 = FcStrCaseWalkerNext (&w2);
+	c1 = FcStrCaseWalkerNext (&w1, NULL);
+	c2 = FcStrCaseWalkerNext (&w2, NULL);
 	if (!c1 || (c1 != c2))
 	    break;
     }
@@ -464,6 +382,75 @@ FcStrContainsWord (const FcChar8 *s1, const FcChar8 *s2)
     return 0;
 }
 
+/*
+ * returns the number of strings (ignoring delimitors and case) being matched
+ */
+
+int
+FcStrMatchIgnoreCaseAndDelims (const FcChar8 *s1, const FcChar8 *s2, const FcChar8 *delims)
+{
+    FcCaseWalker    w1, w2;
+    FcChar8	    c1, c2;
+
+    if (s1 == s2) return 0;
+
+    FcStrCaseWalkerInit (s1, &w1);
+    FcStrCaseWalkerInit (s2, &w2);
+
+    for (;;)
+    {
+	c1 = FcStrCaseWalkerNext (&w1, (const char *)delims);
+	c2 = FcStrCaseWalkerNext (&w2, (const char *)delims);
+	if (!c1 || (c1 != c2))
+	    break;
+    }
+    return w1.src - s1 - 1;
+}
+
+FcBool
+FcStrGlobMatch (const FcChar8 *glob,
+		const FcChar8 *string)
+{
+    FcChar8	c;
+
+    while ((c = *glob++))
+    {
+	switch (c) {
+	case '*':
+	    /* short circuit common case */
+	    if (!*glob)
+		return FcTrue;
+	    /* short circuit another common case */
+	    if (strchr ((char *) glob, '*') == 0)
+	    {
+		size_t l1, l2;
+
+		l1 = strlen ((char *) string);
+		l2 = strlen ((char *) glob);
+		if (l1 < l2)
+		    return FcFalse;
+		string += (l1 - l2);
+	    }
+	    while (*string)
+	    {
+		if (FcStrGlobMatch (glob, string))
+		    return FcTrue;
+		string++;
+	    }
+	    return FcFalse;
+	case '?':
+	    if (*string++ == '\0')
+		return FcFalse;
+	    break;
+	default:
+	    if (*string++ != c)
+		return FcFalse;
+	    break;
+	}
+    }
+    return *string == '\0';
+}
+
 const FcChar8 *
 FcStrStrIgnoreCase (const FcChar8 *s1, const FcChar8 *s2)
 {
@@ -480,12 +467,12 @@ FcStrStrIgnoreCase (const FcChar8 *s1, const FcChar8 *s2)
     FcStrCaseWalkerInit (s1, &w1);
     FcStrCaseWalkerInit (s2, &w2);
 
-    c2 = FcStrCaseWalkerNext (&w2);
+    c2 = FcStrCaseWalkerNext (&w2, NULL);
 
     for (;;)
     {
 	cur = w1.src;
-	c1 = FcStrCaseWalkerNext (&w1);
+	c1 = FcStrCaseWalkerNext (&w1, NULL);
 	if (!c1)
 	    break;
 	if (c1 == c2)
@@ -496,8 +483,8 @@ FcStrStrIgnoreCase (const FcChar8 *s1, const FcChar8 *s2)
 
 	    for (;;)
 	    {
-		c1t = FcStrCaseWalkerNext (&w1t);
-		c2t = FcStrCaseWalkerNext (&w2t);
+		c1t = FcStrCaseWalkerNext (&w1t, NULL);
+		c2t = FcStrCaseWalkerNext (&w2t, NULL);
 
 		if (!c2t)
 		    return cur;
@@ -783,7 +770,6 @@ FcStrBufDestroy (FcStrBuf *buf)
 {
     if (buf->allocated)
     {
-	FcMemFree (FC_MEM_STRBUF, buf->size);
 	free (buf->buf);
 	FcStrBufInit (buf, 0, 0);
     }
@@ -800,7 +786,6 @@ FcStrBufDone (FcStrBuf *buf)
 	ret = malloc (buf->len + 1);
     if (ret)
     {
-	FcMemAlloc (FC_MEM_STRING, buf->len + 1);
 	memcpy (ret, buf->buf, buf->len);
 	ret[buf->len] = '\0';
     }
@@ -833,7 +818,6 @@ FcStrBufChar (FcStrBuf *buf, FcChar8 c)
 	if (buf->allocated)
 	{
 	    size = buf->size * 2;
-	    FcMemFree (FC_MEM_STRBUF, buf->size);
 	    new = realloc (buf->buf, size);
 	}
 	else
@@ -851,7 +835,6 @@ FcStrBufChar (FcStrBuf *buf, FcChar8 c)
 	    buf->failed = FcTrue;
 	    return FcFalse;
 	}
-	FcMemAlloc (FC_MEM_STRBUF, size);
 	buf->size = size;
 	buf->buf = new;
     }
@@ -884,6 +867,68 @@ FcStrUsesHome (const FcChar8 *s)
     return *s == '~';
 }
 
+FcChar8 *
+FcStrBuildFilename (const FcChar8 *path,
+		    ...)
+{
+    va_list ap;
+    FcStrSet *sset;
+    FcStrList *list;
+    FcChar8 *s, *ret = NULL, *p;
+    size_t len = 0;
+
+    if (!path)
+	return NULL;
+
+    sset = FcStrSetCreate ();
+    if (!sset)
+	return NULL;
+
+    if (!FcStrSetAdd (sset, path))
+	goto bail0;
+
+    va_start (ap, path);
+    while (1)
+    {
+	s = (FcChar8 *)va_arg (ap, FcChar8 *);
+	if (!s)
+	    break;
+	if (!FcStrSetAdd (sset, s))
+	    goto bail1;
+    }
+    list = FcStrListCreate (sset);
+    while ((s = FcStrListNext (list)))
+    {
+	len += strlen ((const char *)s) + 1;
+    }
+    list->n = 0;
+    ret = malloc (sizeof (FcChar8) * (len + 1));
+    if (!ret)
+	goto bail2;
+    p = ret;
+    while ((s = FcStrListNext (list)))
+    {
+	if (p != ret)
+	{
+	    p[0] = FC_DIR_SEPARATOR;
+	    p++;
+	}
+	len = strlen ((const char *)s);
+	memcpy (p, s, len);
+	p += len;
+    }
+    *p = 0;
+
+bail2:
+    FcStrListDone (list);
+bail1:
+    va_end (ap);
+bail0:
+    FcStrSetDestroy (sset);
+
+    return ret;
+}
+
 FcChar8 *
 FcStrCopyFilename (const FcChar8 *s)
 {
@@ -942,7 +987,6 @@ FcStrDirname (const FcChar8 *file)
     dir = malloc ((slash - file) + 1);
     if (!dir)
 	return 0;
-    FcMemAlloc (FC_MEM_STRING, (slash - file) + 1);
     strncpy ((char *) dir, (const char *) file, slash - file);
     dir[slash - file] = '\0';
     return dir;
@@ -971,9 +1015,12 @@ FcStrCanonAbsoluteFilename (const FcChar8 *s)
     file = malloc (size);
     if (!file)
 	return NULL;
-    FcMemAlloc (FC_MEM_STRING, size);
     slash = NULL;
     f = file;
+#ifdef _WIN32
+    if (*s == '/' && *(s+1) == '/') /* Network path, do not squash // */
+	*f++ = *s++;
+#endif
     for (;;) {
 	if (*s == '/' || *s == '\0')
 	{
@@ -1071,8 +1118,7 @@ FcStrCanonFilename (const FcChar8 *s)
 	FcChar8	cwd[FC_MAX_FILE_LEN + 2];
 	if (getcwd ((char *) cwd, FC_MAX_FILE_LEN) == NULL)
 	    return NULL;
-	strcat ((char *) cwd, "/");
-	full = FcStrPlus (cwd, s);
+	full = FcStrBuildFilename (cwd, s, NULL);
 	file = FcStrCanonAbsoluteFilename (full);
 	FcStrFree (full);
 	return file;
@@ -1087,8 +1133,7 @@ FcStrSetCreate (void)
     FcStrSet	*set = malloc (sizeof (FcStrSet));
     if (!set)
 	return 0;
-    FcMemAlloc (FC_MEM_STRSET, sizeof (FcStrSet));
-    set->ref = 1;
+    FcRefInit (&set->ref, 1);
     set->num = 0;
     set->size = 0;
     set->strs = 0;
@@ -1109,14 +1154,10 @@ _FcStrSetAppend (FcStrSet *set, FcChar8 *s)
 
 	if (!strs)
 	    return FcFalse;
-	FcMemAlloc (FC_MEM_STRSET, (set->size + 2) * sizeof (FcChar8 *));
 	if (set->num)
 	    memcpy (strs, set->strs, set->num * sizeof (FcChar8 *));
 	if (set->strs)
-	{
-	    FcMemFree (FC_MEM_STRSET, (set->size + 1) * sizeof (FcChar8 *));
 	    free (set->strs);
-	}
 	set->size = set->size + 1;
 	set->strs = strs;
     }
@@ -1190,7 +1231,7 @@ FcStrSetAddLangs (FcStrSet *strs, const char *languages)
     while ((next = strchr (p, ':')))
     {
 	len = next - p;
-	len = FC_MIN (len, 128);
+	len = FC_MIN (len, 127);
 	strncpy ((char *) lang, p, len);
 	lang[len] = 0;
 	/* ignore an empty item */
@@ -1200,7 +1241,7 @@ FcStrSetAddLangs (FcStrSet *strs, const char *languages)
 	    if (normalized_lang)
 	    {
 		FcStrSetAdd (strs, normalized_lang);
-		free (normalized_lang);
+		FcStrFree (normalized_lang);
 		ret = FcTrue;
 	    }
 	}
@@ -1212,7 +1253,7 @@ FcStrSetAddLangs (FcStrSet *strs, const char *languages)
 	if (normalized_lang)
 	{
 	    FcStrSetAdd (strs, normalized_lang);
-	    free (normalized_lang);
+	    FcStrFree (normalized_lang);
 	    ret = FcTrue;
 	}
     }
@@ -1241,23 +1282,34 @@ FcStrSetDel (FcStrSet *set, const FcChar8 *s)
     return FcFalse;
 }
 
+/* TODO Make public */
+static FcStrSet *
+FcStrSetReference (FcStrSet *set)
+{
+    if (FcRefIsConst (&set->ref))
+	return set;
+
+    FcRefInc (&set->ref);
+    return set;
+}
+
 void
 FcStrSetDestroy (FcStrSet *set)
 {
-    if (--set->ref == 0)
-    {
-	int	i;
+    int	i;
 
-	for (i = 0; i < set->num; i++)
-	    FcStrFree (set->strs[i]);
-	if (set->strs)
-	{
-	    FcMemFree (FC_MEM_STRSET, (set->size + 1) * sizeof (FcChar8 *));
-	    free (set->strs);
-	}
-	FcMemFree (FC_MEM_STRSET, sizeof (FcStrSet));
-	free (set);
-    }
+    /* We rely on this in FcGetDefaultLangs for caching. */
+    if (FcRefIsConst (&set->ref))
+	return;
+
+    if (FcRefDec (&set->ref) != 1)
+	return;
+
+    for (i = 0; i < set->num; i++)
+	FcStrFree (set->strs[i]);
+    if (set->strs)
+	free (set->strs);
+    free (set);
 }
 
 FcStrList *
@@ -1268,13 +1320,18 @@ FcStrListCreate (FcStrSet *set)
     list = malloc (sizeof (FcStrList));
     if (!list)
 	return 0;
-    FcMemAlloc (FC_MEM_STRLIST, sizeof (FcStrList));
     list->set = set;
-    set->ref++;
+    FcStrSetReference (set);
     list->n = 0;
     return list;
 }
 
+void
+FcStrListFirst (FcStrList *list)
+{
+    list->n = 0;
+}
+
 FcChar8 *
 FcStrListNext (FcStrList *list)
 {
@@ -1287,7 +1344,6 @@ void
 FcStrListDone (FcStrList *list)
 {
     FcStrSetDestroy (list->set);
-    FcMemFree (FC_MEM_STRLIST, sizeof (FcStrList));
     free (list);
 }
 
